@@ -1,5 +1,4 @@
-from Acquisition import IAcquirer, aq_base, aq_inner, \
-                        ImplicitAcquisitionWrapper, aq_chain
+from Acquisition import aq_base, aq_chain
 from ZODB.interfaces import IConnection
 from ZPublisher.BaseRequest import RequestContainer
 from zExceptions import NotFound
@@ -9,24 +8,27 @@ from zope.app.component.hooks import getSite
 from zope.interface import implements, implementer
 from zope.app.keyreference.interfaces import IKeyReference, NotYet
 from zope.app.keyreference.persistent import KeyReferenceToPersistent
-from site import get_root
+from site import get_root, aq_iter
 from utils import test_settable
 from interfaces import UnsettableAttributeError
 from zope.app.container.interfaces import IObjectAddedEvent
 
+try:
+    from Products.CMFCore.FSPythonScript import FSPythonScript
+except:
+    class FSPythonScript(object):
+        pass
+
+
 @adapter(IPersistent)
 @implementer(IConnection)
 def connectionOfPersistent(obj):
     """ zope2 cxn fetcher for wrapped items """
-    cur = obj
-    if IAcquirer.providedBy(obj) or hasattr(obj, '__parent__'):
-        while not getattr(cur, '_p_jar', None):
-            cur = getattr(cur, 'aq_parent', getattr(cur, '__parent__', None))
-            if cur is None:
-                return None
-        return cur._p_jar
-    else:
-        raise TypeError("%s not acquisition wrapped" %obj)
+    for parent in aq_iter(obj):
+        conn = getattr(parent, '_p_jar', None)
+        if conn is not None:
+            return conn
+
 
 @adapter(IPersistent, IObjectAddedEvent)
 def add_object_to_connection(ob, event):
@@ -64,7 +66,8 @@ class KeyReferenceToPersistent(KeyReferenceToPersistent):
 
             # DirectoryViews do not allow setting of _p_oid
             # will cause transaction errors
-            if not test_settable(self.object, '_p_oid'):
+            if (not test_settable(self.object, '_p_oid')
+                or isinstance(self.object, FSPythonScript)):
                 raise UnsettableAttributeError(wrapped_obj)
             connection.add(self.object)
 
@@ -87,7 +90,7 @@ class KeyReferenceToPersistent(KeyReferenceToPersistent):
         if self.path is None:
             return self.object
         try:
-            obj = self.root.restrictedTraverse(self.path)
+            obj = self.root.unrestrictedTraverse(self.path)
         except NotFound:
             return self.object
         chain = aq_chain(obj)
@@ -96,7 +99,8 @@ class KeyReferenceToPersistent(KeyReferenceToPersistent):
         if not len(chain) or not isinstance(chain[-1], RequestContainer):
             site = getSite()
             site_chain = aq_chain(site)
-            if len(site_chain) and isinstance(site_chain[-1], RequestContainer):
+            if len(site_chain) and isinstance(site_chain[-1],
+                                              RequestContainer):
                 req = site_chain[-1]
                 new_obj = req
                 # rebuld the chain with the request at the bottom
@@ -116,11 +120,13 @@ class KeyReferenceToPersistent(KeyReferenceToPersistent):
     def __cmp__(self, other):
         if self.key_type_id == other.key_type_id:
             try:
-                other = (IConnection(other.object).db().database_name, other.object._p_oid)
+                other = (IConnection(other.object).db().database_name,
+                         other.object._p_oid)
             except TypeError:
                 other = None
             return cmp(
-                (IConnection(self.object).db().database_name,  self.object._p_oid),
+                (IConnection(self.object).db().database_name,
+                 self.object._p_oid),
                 other,
                 )
 
